package com.pixelmintdrop.model

import android.util.Log

/**
 * Represents the game board and manages piece placement and collision detection
 */
class GameBoard {
    companion object {
        const val WIDTH = 10
        const val HEIGHT = 20
        const val INITIAL_DROP_INTERVAL = 1000L // 1 second
        const val MIN_DROP_INTERVAL = 100L // 0.1 seconds
    }

    // Board state
    private val board = Array(HEIGHT) { Array(WIDTH) { false } }
    private var currentPiece: GamePiece? = null
    private var nextPiece: GamePiece? = null
    
    // Game state
    var dropInterval = INITIAL_DROP_INTERVAL
        private set
    
    val width: Int = WIDTH
    val height: Int = HEIGHT

    fun reset() {
        // Clear the board
        for (y in 0 until HEIGHT) {
            for (x in 0 until WIDTH) {
                board[y][x] = false
            }
        }
        
        // Reset pieces
        currentPiece = null
        nextPiece = null
        
        // Reset drop interval
        dropInterval = INITIAL_DROP_INTERVAL
    }

    fun spawnNewPiece(): GamePiece {
        // Get the next piece or create a new one if none exists
        currentPiece = nextPiece ?: createRandomPiece()
        nextPiece = createRandomPiece()
        
        // Position the piece at the top center of the board
        currentPiece?.let { piece ->
            piece.x = (WIDTH - piece.getWidth()) / 2
            piece.y = 0
        }
        
        return currentPiece!!
    }

    fun getNextPiece(): GamePiece? = nextPiece

    fun getCurrentPiece(): GamePiece? = currentPiece

    fun hasBlockAt(x: Int, y: Int): Boolean {
        if (x < 0 || x >= WIDTH || y < 0 || y >= HEIGHT) return false
        return board[y][x]
    }

    fun wouldCollide(piece: GamePiece, newX: Int, newY: Int): Boolean {
        val width = piece.getWidth()
        val height = piece.getHeight()
        
        for (y in 0 until height) {
            for (x in 0 until width) {
                if (piece.isBlockAt(x, y)) {
                    val boardX = newX + x
                    val boardY = newY + y
                    
                    // Check board boundaries
                    if (boardX < 0 || boardX >= WIDTH || boardY >= HEIGHT) {
                        return true
                    }
                    
                    // Check collision with placed blocks
                    if (boardY >= 0 && board[boardY][boardX]) {
                        return true
                    }
                }
            }
        }
        
        return false
    }

    fun movePiece(piece: GamePiece, dx: Int, dy: Int): Boolean {
        val newX = piece.x + dx
        val newY = piece.y + dy
        
        if (!wouldCollide(piece, newX, newY)) {
            piece.x = newX
            piece.y = newY
            return true
        }
        
        return false
    }

    fun rotatePiece(piece: GamePiece): Boolean {
        // Save current state
        val originalRotation = piece.rotation
        
        // Try to rotate
        piece.rotate()
        
        // Check if new position is valid
        if (wouldCollide(piece, piece.x, piece.y)) {
            // If not valid, try wall kicks
            val kicks = arrayOf(
                Pair(1, 0),  // Try moving right
                Pair(-1, 0), // Try moving left
                Pair(0, -1), // Try moving up
                Pair(2, 0),  // Try moving right 2
                Pair(-2, 0)  // Try moving left 2
            )
            
            for ((kickX, kickY) in kicks) {
                if (!wouldCollide(piece, piece.x + kickX, piece.y + kickY)) {
                    piece.x += kickX
                    piece.y += kickY
                    return true
                }
            }
            
            // If no wall kick worked, revert rotation
            piece.rotation = originalRotation
            return false
        }
        
        return true
    }

    fun lockPiece(piece: GamePiece) {
        val width = piece.getWidth()
        val height = piece.getHeight()
        
        // Place the piece on the board
        for (y in 0 until height) {
            for (x in 0 until width) {
                if (piece.isBlockAt(x, y)) {
                    val boardX = piece.x + x
                    val boardY = piece.y + y
                    if (boardY >= 0 && boardY < HEIGHT && boardX >= 0 && boardX < WIDTH) {
                        board[boardY][boardX] = true
                    }
                }
            }
        }
        
        // Clear any completed lines
        clearLines()
    }

    private fun clearLines() {
        var linesCleared = 0
        var y = HEIGHT - 1
        
        while (y >= 0) {
            if (isLineFull(y)) {
                // Move all lines above down
                for (moveY in y downTo 1) {
                    for (x in 0 until WIDTH) {
                        board[moveY][x] = board[moveY - 1][x]
                    }
                }
                // Clear top line
                for (x in 0 until WIDTH) {
                    board[0][x] = false
                }
                linesCleared++
            } else {
                y--
            }
        }
        
        // Increase speed based on lines cleared
        if (linesCleared > 0) {
            dropInterval = (dropInterval * 0.95).toLong().coerceAtLeast(MIN_DROP_INTERVAL)
        }
    }

    private fun isLineFull(y: Int): Boolean {
        for (x in 0 until WIDTH) {
            if (!board[y][x]) return false
        }
        return true
    }

    private fun createRandomPiece(): GamePiece {
        return GamePiece(GamePieceType.values().random())
    }
} 